  Member

!*
!*  Function Prototypes
!*

  map
    ! Local map, can be empty but MUST be present
    MODULE('Win32')
      GetEnvironmentVariable(*Cstring, *Cstring, Long), Long, Pascal, Raw, name('GetEnvironmentVariableA')
      SetEnvironmentVariable(*Cstring, *Cstring), Bool, Pascal, Raw
    END
  end

Include('HeaderSort.inc')
Include('KeyCodes.clw'), Once

HeaderSortClass.Construct  Procedure
Code

HeaderSortClass.Destruct  Procedure
Code
  Self.Kill
!
!  Initialize the headersort fields and queues.
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!    SortAscIn   - The symbol used to signify ascending sort
!    SortDescIn  - The symbol used to signify descending sort
!    PSFieldIn   - Primary sort field when no sort is selected
!    PSOrderIn   - Primary sort order used with primary sort field
!    SortIn      - Flag for if sort is saved or not
!
HeaderSortClass.Init  Procedure(CurrentUse, CurrentFrom, SortAscIn, SortDescIn, PSFieldIn, PSOrderIn, SortIn)
Code
!
!  Save the listbox's original format
!    Pre-set the AskRestoreFlag to True
!
  Self.OrigFormat = CurrentUse{Prop:Format}
  Self.AskRestoreFlag = True
!
!  Set initial values of variables and create sort queues
!
  Self.SortAsc  = SortAscIn
  Self.SortDesc = SortDescIn
  Self.SortSaveFlag = SortIn
  Self.SortQueue &= New(SortQueueType)
  Self.SortDisplay &= New(SortDisplayType)
  Self.PrimarySortField = PSFieldIn
  Self.PrimarySortOrder = PSOrderIn
!
!  Fill the sort queues
!
  Self.FillSort(CurrentUse, CurrentFrom)
!
!  Sort the display queue
!
  Self.ReSort(CurrentUse, CurrentFrom, 1)
!
!  Get the location used to save the sort ini
!
  Self.CreateFlag = True
  Self.ev.Name   = 'HOMEDRIVE'
  Self.ev.Value  = ''
  Self.ev.Length = 32767
  If Not GetEnvironmentVariable(Self.ev.Name, Self.ev.Value, Self.ev.Length) Then
    Self.HomePath = 'c:'
  Else
    Self.HomePath = Clip(Left(Self.ev.Value))
  End
  Self.ev.Name   = 'HOMEPATH'
  Self.ev.Value  = ''
  Self.ev.Length = 32767
  If Not GetEnvironmentVariable(Self.ev.Name, Self.ev.Value, Self.ev.Length) Then
    Self.HomePath = Clip(Self.HomePath) & '\'
  Else
    Self.HomePath = Clip(Self.HomePath) & Clip(Left(Self.ev.Value)) & '\'
  End
  Return

!
!  Clean up the created queues
!
HeaderSortClass.Kill Procedure
Code
  If Self.CreateFlag Then
    Free(Self.SortDisplay)
    Dispose(Self.SortDisplay)
    Free(Self.SortQueue)
    Dispose(Self.SortQueue)
  End
  Return

!
!  Set the AskRestoreFlag
!
HeaderSortClass.SetRestoreFlag  Procedure(RestoreIn)
Code
  Self.AskRestoreFlag = RestoreIn
  Return

!
!  Fill the sort queue with all of the fields in the list
!    There is no field total count so have to read each consecutive field
!    until none exist
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
HeaderSortClass.FillSort  Procedure(CurrentUse, CurrentFrom)
  CODE
  Free(Self.SortQueue)
  Self.SortI = 1
  Loop While CurrentUse{PropList:Exists, Self.SortI}
    Clear(Self.SortQueue)
    Self.SortQueue.FieldName = Who(CurrentFrom, CurrentUse{PropList:FieldNo, Self.SortI})
    Self.SortQueue.FieldDisp = CurrentUse{PropList:Header, Self.SortI}
    Self.SortQueue.QueueNum  = CurrentUse{PropList:FieldNo, Self.SortI}
    Add(Self.SortQueue)
    Self.SortI += 1
  End
  Return

!
!  Load the saved sort for this table
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!    SectionIn   - The identyfying name for the table
!
HeaderSortClass.LoadSort  Procedure(CurrentUse, CurrentFrom, SectionIn)
SortFieldCount Long
SortFields     String(100)
ColumnSets     String(1000)
I              Long
J              Long
StrPos         Long
LoopFlag       Byte
  CODE
!
!  Only process if there is a section name and if the sort is to be saved
!
  If Clip(Left(SectionIn)) > '' And Self.SortSaveFlag Then
!
!  There is a lot of error checking in this area.  Some users are rather prone to
!    modifying ini files just because they think they can.  This section helps to
!    repair the different changes they might have made.
!    If any variations are found, then ColumnSets will be cleared.  This will
!      help make sure that the column formats are still ok.
!
!  Get the number of fields an field list
!
    SortFieldCount = GetIni(Clip(Left(SectionIn)), 'SortFieldCount', 0, Clip(Self.HomePath) & 'Claims.ini')
    SortFields     = GetIni(Clip(Left(SectionIn)), 'SortFields', 0, Clip(Self.HomePath) & 'Claims.ini')
    ColumnSets     = GetIni(Clip(Left(SectionIn)), 'ColumnSets', '', Clip(Self.HomePath) & 'Claims.ini')
 !
 ! Check if number of sorted fields are within acceptable ranges
 !
    If SortFieldCount > 0 Then
      If SortFieldCount > 33 Then
        Message('Saved Sort Greater Than 33 Fields - Truncating')
        SortFieldCount = 33
        ColumnSets = ''
      End
      StrPos   = 0
      I        = 0
      LoopFlag = True
!
!  Count the number of entries in the sortfields
!
      Loop While LoopFlag
        StrPos += 1
        StrPos = InString(',', SortFields, 1, StrPos)
        If StrPos > 0 Then
          I += 1
        Else
          LoopFlag = False
        End
      End
      If I > 0 Then
!
!  Empty the sort display queue
!
        Free(Self.SortDisplay)
        I += 1
!
!  There are 3 values per field, if extra fields then truncate to 3 per field
!
        If I % 3 > 0 Then
          Message('Saved Sort Data Error - Truncating To ' & Int(I / 3) & ' Fields')
          SortFieldCount = Int(I / 3)
          ColumnSets = ''
        End
!
!  The saved field list is too short.  Only read what is available
!
        If SortFieldCount > Int(I / 3) Then
          Message('Saved Sort Data Unexpectedly Truncated - Reading Only ' & Int(I / 3) & ' Fields')
          SortFieldCount = Int(I / 3)
          ColumnSets = ''
        End
        If SortFieldCount > 0 Then
          I = 0
          StrPos = 1
          LoopFlag = True
          Loop While LoopFlag
            I += 1
            Clear(Self.SortDisplay)
!
!  Load the field number for the sorted field
!
            J = InString(',', SortFields, 1, StrPos)
            If J = 0 Then
              J = Len(Clip(SortFields)) + 1
            End
            Self.SortDisplay.FieldNum = Sub(SortFields, StrPos, J - StrPos)
!
!  Check for a valid field number in the list
!
            If Not CurrentUse{PropList:Exists, Self.SortDisplay.FieldNum} Then
              StrPos = -1
              LoopFlag = False
              ColumnSets = ''
            Else
!
!  Load the corresponding queue field for the list field
!
              StrPos = J + 1
              J = InString(',', SortFields, 1, StrPos)
              If J = 0 Then
                J = Len(Clip(SortFields)) + 1
              End
              Self.SortDisplay.QueueNum = CurrentUse{PropList:FieldNo, Self.SortDisplay.FieldNum}
!
!  Load the sort direction used for the field
!    Reset the sort value if outside the allowable selection range
!
              StrPos = J + 1
              J = InString(',', SortFields, 1, StrPos)
              If J = 0 Then
                J = Len(Clip(SortFields)) + 1
              End
              Self.SortDisplay.Sort = Sub(SortFields, StrPos, J - StrPos)
              If Self.SortDisplay.Sort > 2 Then
                Self.SortDisplay.Sort = 2
              End
              If Self.SortDisplay.Sort < 1 Then
                Self.SortDisplay.Sort = 1
              End
              StrPos = J + 1
!
!  Add the field to the display sort queue
!
              Add(Self.SortDisplay)
            End
!
!  If the field list is too long, stop when the proper number have been read only.
!
            If I => SortFieldCount Then
              LoopFlag = False
            End
          End
!
!  Make sure the display sort queue is empty if no proper field values loaded.
!    This also makes sure the primary sort gets used
!
          If StrPos < 0 Then
            Free(Self.SortDisplay)
          End
        End
      End
    End
!
!  Set the saved column format if columnSets is not blank.
!
    If Clip(ColumnSets) > '' Then
      StrPos   = 0
      I        = 0
      J        = 0
      LoopFlag = True
!
!  Move through the number of entries in the ColumnSets
!
      Loop While LoopFlag
        I      += 1
        StrPos  = J + 1
!
!  Check for a valid column number
!
        If CurrentUse{PropList:Exists, I} Then
!
!  Get next field position in string
!
          StrPos = InString(',', ColumnSets, 1, StrPos)
!
!  Check for end of string and turn off loop for end.
!    Continue processing for the last field though.
!
          If StrPos = 0 Then
            StrPos   = Len(Clip(ColumnSets)) + 1
            LoopFlag = False
          End
!
!  Check for a number to be loaded as column width
!    Load the saved width for the column
!
          If Numeric(Sub(ColumnSets, J + 1, StrPos - J - 1)) Then
            CurrentUse{PropList:Width, I} = Sub(ColumnSets, J + 1, StrPos - J - 1)
          End
!
!  Update the start position for the next field.
!
          J = StrPos
        Else
!
!  Column does not exist.  Turn off the loop and exit.
!
          LoopFlag = False
        End
      End
    End

!
!  Sort the queue
!
    Self.ReSort(CurrentUse, CurrentFrom, 1)
  End

!
!  Save the sort for this table
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!    SectionIn   - The identyfying name for the table
!
HeaderSortClass.SaveSort  Procedure(CurrentUse, CurrentFrom, SectionIn)
SortFieldCount Long
SortFields     String(100)
ColumnSets     String(500)
I              Long
  CODE
!
!  Only process if there is a section name and if the sort is to be saved
!
  If Clip(Left(SectionIn)) > '' And Self.SortSaveFlag Then
    SortFieldCount = Records(Self.SortDisplay)
!
!  Check if there is a sort currently set for the queue.
!
    If SortFieldCount > 0 Then
      SortFields = ''
!
!  Loop through the sort fields and build the sort string to save.
!    The string will save the field number, queue number, and sort direction
!    for each field in the sort queue
!
      Loop I = 1 To Records(Self.SortDisplay)
        Get(Self.SortDisplay, I)
        If I > 1 Then
          SortFields = Clip(SortFields) & ','
        End
        SortFields = Clip(SortFields) & Self.SortDisplay.FieldNum
        SortFields = Clip(SortFields) & ',' & Self.SortDisplay.QueueNum
        SortFields = Clip(SortFields) & ',' & Self.SortDisplay.Sort
      End
!
!  Loop through the columns and build the column width string to save.
!    The string will have all the currently set column widths.
!
      I = 1
      ColumnSets = ''
      Loop While CurrentUse{PropList:Exists, I}
        If I > 1 Then
          ColumnSets = Clip(ColumnSets) & ','
        End
        ColumnSets = Clip(ColumnSets) & CurrentUse{PropList:Width, I}
        I += 1
      End
!
!  Save the sort to the ini file.
!
      PutIni(Clip(Left(SectionIn)), 'SortFieldCount', SortFieldCount, Clip(Self.HomePath) & 'Claims.ini')
      PutIni(Clip(Left(SectionIn)), 'SortFields', SortFields, Clip(Self.HomePath) & 'Claims.ini')
      PutIni(Clip(Left(SectionIn)), 'ColumnSets', ColumnSets, Clip(Self.HomePath) & 'Claims.ini')
    Else
!
!  No sort set - make sure the saved sort is cleared.
!
      PutIni(Clip(Left(SectionIn)), 'SortFieldCount', 0, Clip(Self.HomePath) & 'Claims.ini')
      PutIni(Clip(Left(SectionIn)), 'SortFields', '', Clip(Self.HomePath) & 'Claims.ini')
      PutIni(Clip(Left(SectionIn)), 'ColumnSets', CurrentUse{Prop:Format}, Clip(Self.HomePath) & 'Claims.ini')
    End
  Else
!
!  Save sort not selected.  Make sure there are no prior sorts saved for this queue.
!
    If Clip(Left(SectionIn)) > '' Then
      PutIni(Clip(Left(SectionIn)), 'SortFieldCount', 0, Clip(Self.HomePath) & 'Claims.ini')
      PutIni(Clip(Left(SectionIn)), 'SortFields', '', Clip(Self.HomePath) & 'Claims.ini')
    End
  End

!
!  Sort the queue
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!    TopFlag     - Reset the record pointer to the top of the list
!
HeaderSortClass.ReSort Procedure(CurrentUse, CurrentFrom, TopFlag)
I  Long
J  Byte
  CODE
  J = True
  I = 0
!
!  Re-set the display title for the headers for each field on the table
!
  Loop While J
    I += 1
    Get(Self.SortQueue, I)
    If CurrentUse{PropList:Exists, I} Then
      CurrentUse{PropList:Header, I} = Clip(Self.SortQueue.FieldDisp)
    Else
      J = False
    End
  End
  Self.SortLine = ''
!
!  Loop through the selected sort queue and build the sort string
!
  Loop Self.SortI = 1 to Records(Self.SortDisplay)
    Get(Self.SortDisplay, Self.SortI)
    If Not ErrorCode() Then
      Get(Self.SortQueue, Self.SortDisplay.FieldNum)
      If Self.SortI > 1 Then
        Self.SortLine = Clip(Self.SortLine) & ','
      End
!
!  Set the ascending or descending sort order.
!
      If Self.SortDisplay.Sort = 1 Then
        Self.SortLine = Clip(Self.SortLine) & ' ' & '+'
        CurrentUse{PropList:Header, Self.SortDisplay.FieldNum} = Clip(Self.SortQueue.FieldDisp) & Clip(Left(Self.SortAsc))
      Else
        Self.SortLine = Clip(Self.SortLine) & ' ' & '-'
        CurrentUse{PropList:Header, Self.SortDisplay.FieldNum} = Clip(Self.SortQueue.FieldDisp) & Clip(Left(Self.SortDesc))
      End
!
!  Set the field name to be sorted upon.
!
      Self.SortLine = Clip(Self.SortLine) & Clip(Self.SortQueue.FieldName)
    End
  End
!
!  If the sort string is empty, then default to the primary sort
!
  If Clip(Self.SortLine) = '' Then
    Get(Self.SortQueue, Self.PrimarySortField)
    Self.SortLine = Self.PrimarySortOrder & Clip(Self.SortQueue.FieldName)
  End
!
!  Sort the queue.
!
  Sort(CurrentFrom, Self.SortLine)
!
! Reset the record position to the last selected record.
!
  If Not TopFlag Then
    Self.SortLine = ''
    Loop Self.SortI = 1 To Records(Self.SortQueue)
      Get(Self.SortQueue, Self.SortI)
      If Self.SortI > 1 Then
        Self.SortLine = Clip(Self.SortLine) & ','
      End
      Self.SortLine = Clip(Self.SortLine) & Clip(Self.SortQueue.FieldName)
    End
    Get(CurrentFrom, Self.SortLine)
  Else
    Get(CurrentFrom, 1)
  End
  CurrentUse{Prop:Selected} = Pointer(CurrentFrom) !Position(CurrentFrom)
  Post(Event:NewSelection, CurrentUse)
!
!  Adjust the slider if needed.
! 
  Self.ThumbPlace(CurrentUse, CurrentFrom)
  Return

!
!  Process the alert keys for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
HeaderSortClass.ListAlert Procedure(CurrentUse, CurrentFrom)
FoundRecord   Long
StartRecord   Long
EndRecord     Long
CheckRecord   Long
SearchFlag    Byte
TopFlag       Byte
  CODE
  SetCursor(Cursor:Wait)
  TopFlag = False
!
!  Check if this is a double-click on the headers.
!  If so, then this is for changing the current sort.
!
  If CurrentUse{PropList:MouseDownRow} = 0 And KeyCode() = MouseLeft2 Then
!
!  Get the field clicked upon.
!
    Self.SortJ = CurrentUse{PropList:MouseDownField}
    Get(Self.SortQueue, Self.SortJ)
!
!  Check if the selected field is already being used for sorting the queue
!
    Clear(Self.SortDisplay)
    Self.SortDisplay.FieldNum = Self.SortJ
    Get(Self.SortDisplay, Self.SortDisplay.FieldNum)
    If Not ErrorCode() And Self.SortDisplay.FieldNum = Self.SortJ Then
!
!  Adjust the sorted field to the next option
!    Ascending ==> Descending ==> No Sort
!    Adjust the header with the sort descriptor
!
      If Self.SortDisplay.Sort = 2 Then
        Self.SortDisplay.Sort = 0
        CurrentUse{PropList:Header, Self.SortJ} = Clip(Self.SortQueue.FieldDisp)
        Delete(Self.SortDisplay)
      Else
        Self.SortDisplay.Sort += 1
        CurrentUse{PropList:Header, Self.SortJ} = Clip(Self.SortQueue.FieldDisp) & Clip(Left(Self.SortDesc))
        Put(Self.SortDisplay)
      End
    Else
!
!  Add the selected field to the end of the sort queue.  Set as ascending.
!    Adjust the header with the sort descriptor
!
      Clear(Self.SortDisplay)
      Self.SortDisplay.FieldNum = Self.SortJ
      Self.SortDisplay.QueueNum = Self.SortQueue.QueueNum
      Self.SortDisplay.Sort = 1
      Add(Self.SortDisplay)
      CurrentUse{PropList:Header, Self.SortJ} = Clip(Self.SortQueue.FieldDisp) & Clip(Left(Self.SortAsc))
    End
!
!  Build the sortline and resort the queue.
!    This is similar to the resort procedure so I will adjust this later
!    to use that.
!
    Self.SortLine = ''
!
!  Loop through the fields in the sort queue and set their
!    sort descriptor and fieldname
!
    Loop Self.SortI = 1 to Records(Self.SortDisplay)
      Get(Self.SortDisplay, Self.SortI)
      If Not ErrorCode() Then
        Get(Self.SortQueue, Self.SortDisplay.FieldNum)
        If Self.SortI > 1 Then
          Self.SortLine = Clip(Self.SortLine) & ','
        End
        If Self.SortDisplay.Sort = 1 Then
          Self.SortLine = Clip(Self.SortLine) & ' ' & '+'
        Else
          Self.SortLine = Clip(Self.SortLine) & ' ' & '-'
        End
        Self.SortLine = Clip(Self.SortLine) & Clip(Self.SortQueue.FieldName)
      End
    End
!
!  If no sort fields, then default to the primary sort.
!
    If Clip(Self.SortLine) = '' Then
      Get(Self.SortQueue, Self.PrimarySortField)
      Self.SortLine = Self.PrimarySortOrder & Clip(Self.SortQueue.FieldName)
    End
!
!  Sort the queue.
!
    Sort(CurrentFrom, Self.SortLine)
!
! Reset the record position to the last selected record.
!
    Self.SortLine = ''
    Loop Self.SortI = 1 To Records(Self.SortQueue)
      Get(Self.SortQueue, Self.SortI)
      If Clip(Self.SortLine) > '' Then
        Self.SortLine = Clip(Self.SortLine) & ','
      End
      Self.SortLine = Clip(Self.SortLine) & Clip(Self.SortQueue.FieldName)
      If Self.SortI = 16 Then
        Self.SortI = Records(Self.SortQueue)
      End
    End
    Get(CurrentFrom, Self.SortLine)
    CurrentUse{Prop:Selected} = Pointer(CurrentFrom) !Position(CurrentFrom)
  Else
!
!  Process the alert key
!
!  Retrieve the last field used to set the sort.
!    This is the active locator field.
!
    Clear(Self.SortDisplay)
    If Records(Self.SortDisplay) = 0 Then
      Self.SortDisplay.FieldNum = Self.PrimarySortField
    Else
      Get(Self.SortDisplay, Records(Self.SortDisplay))
    End
    Get(Self.SortQueue, Self.SortDisplay.FieldNum)
!
!  Check for specific keys to be processed here.
!
    Case KeyCode()
      Of BSKey
 !
 !  Backspace key -
 !    remove the last entered character from the locator field
 !
        If Self.LocatorLength > 0 Then
          Self.LocatorLength -= 1
          Self.LocatorValue = SUB(Self.LocatorValue, 1, Self.LocatorLength)
          Clear(CurrentFrom)
          If Self.LocatorLength > 0 Then
            Self.SearchFlag = True
          Else
            CurrentUse{Prop:Selected} = 1
            TopFlag = True
          End
        End
      Of SpaceKey
 !
 !  Space key -
 !    Add a space to the locator field
 !
        Self.LocatorValue = SUB(Self.LocatorValue ,1 ,Self.LocatorLength) & ' '
        Self.LocatorLength += 1
        Self.SearchFlag = True
      Of DownKey
 !
 !  Down Arrrow key -
 !    Clear the Locator and select the next record in the table.
 !
        Self.LocatorValue = ''
        Self.LocatorLength = 0
      Of HomeKey
 !
 !  Home key -
 !    Clear the Locator and select the first record in the table.
 !
        Self.LocatorValue = ''
        Self.LocatorLength = 0
        CurrentUse{Prop:Selected} = 1
        TopFlag = True
      Of EscKey
 !
 !  Escape key -
 !    Clear the selected sorts and reset the headers in the table.
 !    Clear the Locator and select the first record in the table.
 !
        Loop Self.SortI = 1 To Records(Self.SortDisplay)
          Get(Self.SortDisplay, Self.SortI)
          Clear(Self.SortQueue)
          Get(Self.SortQueue, Self.SortDisplay.FieldNum)
          CurrentUse{PropList:Header, Self.SortDisplay.FieldNum} = Clip(Self.SortQueue.FieldDisp)
        End
        Free(Self.SortDisplay)
        Self.LocatorValue = ''
        Self.LocatorLength = 0
        CurrentUse{Prop:Selected} = 1
        TopFlag = True
      Of DeleteKey
 !
 !  Delete key -
 !    Clear the selected sorts and reset the headers in the table.
 !    Clear the Locator and select the first record in the table.
 !
        Loop Self.SortI = 1 To Records(Self.SortDisplay)
          Get(Self.SortDisplay, Self.SortI)
          Clear(Self.SortQueue)
          Get(Self.SortQueue, Self.SortDisplay.FieldNum)
          CurrentUse{PropList:Header, Self.SortDisplay.FieldNum} = Clip(Self.SortQueue.FieldDisp)
        End
        Free(Self.SortDisplay)
        Self.LocatorValue = ''
        Self.LocatorLength = 0
        CurrentUse{Prop:Selected} = 1
        TopFlag = True
      Of MouseRight
 !
 !  Right mouse click
 !    Drop down an option to restore the table to original format.
 !
        If CurrentUse{PropList:MouseDownRow} = 0 Then
          If Self.AskRestoreFlag Then
            Execute Popup('Reload Original Table Format|Cancel')
              CurrentUse{Prop:Format} = Self.OrigFormat
            End
          End
        End
      Else
!
!  Not an action key that was pressed.
!    So, it can be processed as a key for the locator.
!
!  check if the key is a visible character
!    then add to the locator field if it is.
!
        IF Clip(CHR(KEYCHAR())) > ''
          Self.LocatorValue = SUB(Self.LocatorValue, 1, Self.LocatorLength) & CHR(KEYCHAR())
          Self.LocatorLength += 1
          Self.SearchFlag = True
        End
    End
!
!  Check if a search needs to be performed.
!
    If Self.SearchFlag Then
      Self.SearchFlag = False
!
!  Check if only 1 sort field and if it is a string field.
!    With a single sorted string field a binary search can be used
!    to make a fast locator for the queue.
!
      If Records(Self.SortDisplay) < 2 And IsString(What(CurrentFrom, Self.SortQueue.QueueNum)) Then
        FoundRecord = 0
        StartRecord = 1
        EndRecord = Records(CurrentFrom)
!
!  Search through the queue for the first record that matches the locator
!    and all potential records have been searched.
!
        Loop While StartRecord <= EndRecord
          CheckRecord = StartRecord + ((EndRecord - StartRecord) /2)  + .5
          Get(CurrentFrom, CheckRecord)
          If Upper(Sub(Self.LocatorValue, 1, Self.LocatorLength)) = Upper(Sub(What(CurrentFrom, Self.SortQueue.QueueNum), 1, Self.LocatorLength)) Then
!
!  Matching record found.
!    Move the end record pointer up to the check record pointer.
!    There may be more records before finding the first matching record.
!
            FoundRecord = CheckRecord
            If EndRecord > CheckRecord Then
              EndRecord = CheckRecord
            Else
              EndRecord -= 1
            End
          Else
            If Upper(Sub(Self.LocatorValue, 1, Self.LocatorLength)) < Upper(Sub(What(CurrentFrom, Self.SortQueue.QueueNum), 1, Self.LocatorLength)) Then
!
!  Wanted record is before the current check record.  Move up the end record.
!
              If EndRecord > CheckRecord Then
                EndRecord = CheckRecord
              Else
                EndRecord -= 1
              End
            Else
!
!  Wanted record is after the current check record.  Move down the start record.
!
              If StartRecord < CheckRecord Then
                StartRecord = CheckRecord
              Else
                StartRecord += 1
              End
            End
          End
        End
      Else
!
!  There are either multiple sort fields or the sort field is not a string.
!    Can not currently do a binary search,  The field might not be in proper 
!    sorted order.  So, have to go through the queue one record at a time
!    until the first record is found that matches the locator.
!
!  Loop through the records until the first record is found that matches
!    the locator.  Compare both by held value and formatted value.
!
          FoundRecord = 0
          StartRecord = 1
          EndRecord = Records(CurrentFrom)
          Loop While FoundRecord = 0 And StartRecord < EndRecord
            Get(CurrentFrom, StartRecord)
            If (Upper(Sub(Self.LocatorValue, 1, Self.LocatorLength)) = Upper(Sub(Left(What(CurrentFrom, Self.SortQueue.QueueNum)), 1, Self.LocatorLength))) Or (Upper(Sub(Self.LocatorValue, 1, Self.LocatorLength)) = Upper(Sub(Left(Format(What(CurrentFrom, Self.SortQueue.QueueNum), CurrentUse{PropList:Picture, Self.SortDisplay.FieldNum})), 1, Self.LocatorLength))) Then
              FoundRecord = StartRecord
            Else
              StartRecord += 1
            End
          End
      End
!
!  If a record was found that matches the locator, go to that record.
!    Otherwise, go to the first record in the queue.
!
      If FoundRecord Then
        CurrentUse{Prop:Selected} = FoundRecord
      Else
        CurrentUse{Prop:Selected} = 1
        TopFlag = True
      End
    Else
!
!  No locator searching needed.
!    Just reset the sort to make sure the display is correct.
!
      Self.ReSort(CurrentUse, CurrentFrom, TopFlag)
    End
  End
!
!  Set the slider for the list to the proper location.
!  Process the selected record as a new selection for the calling program.
!  Reset the cursor.
!
  Self.ThumbPlace(CurrentUse, CurrentFrom)
  Post(Event:NewSelection, CurrentUse)
  SetCursor(Cursor:Arrow)
  Return
