  Member

!*
!*  Function Prototypes
!*

  map
    ! Local map, can be empty but MUST be present
  end

Include('TableHandle.inc')

!
!  Move the thumb placement to the selected record for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
TableHandleKeys.ThumbPlace Procedure(CurrentUse, CurrentFrom)
CurrentPos  Long
NewThumb    Long
  CODE
  If CurrentUse{Prop:Vscroll} Then
    CurrentPos = Choice(CurrentUse)
    If CurrentPos = 1 Then
      NewThumb = 0
    Else
!
!  The help file says that 256 is the endpoint for a thumb control.  But, it 
!    seems that for basic list controls, the endpoint is set to the length of
!    the queue instead.
!
      !NewThumb = Int((Choice(CurrentUse) / Records(CurrentFrom)) * 256) - 1
      NewThumb = Choice(CurrentUse)
    End
    CurrentUse{Prop:VScrollPos} = NewThumb
  End
  Return

!
!  Process the pagedown keys for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
TableHandleKeys.ListPageDown  Procedure(CurrentUse, CurrentFrom)
  CODE
!
!  Move the selected record down 1 full screen of records up to the end of the queue.
!  Adjust the thumb placement.
!
  If CurrentUse{Prop:Selected} + CurrentUse{Prop:Items} > Records(CurrentFrom) Then
    CurrentUse{Prop:Selected} = Records(CurrentFrom)
  Else
    CurrentUse{Prop:Selected} = CurrentUse{Prop:Selected} + CurrentUse{Prop:Items}
  End
  Self.ThumbPlace(CurrentUse, CurrentFrom)
  Post(Event:NewSelection, CurrentUse)
  Return

!
!  Process the pageup keys for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
TableHandleKeys.ListPageUp  Procedure(CurrentUse, CurrentFrom)
  CODE
!
!  Move the selected record up 1 full screen of records up to the top of the queue.
!  Adjust the thumb placement.
!
  CurrentUse{Prop:Selected} = CurrentUse{Prop:Selected} - CurrentUse{Prop:Items}
  If CurrentUse{Prop:Selected} < 1 Then
    CurrentUse{Prop:Selected} = 1
  End
  Self.ThumbPlace(CurrentUse, CurrentFrom)
  Post(Event:NewSelection, CurrentUse)
  Return

!
!  Process the end keys for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
TableHandleKeys.ListScrollBottom  Procedure(CurrentUse, CurrentFrom)
  CODE
!
!  Move the selected record down to the end of the queue.
!  Adjust the thumb placement.
!
  CurrentUse{Prop:Selected} = Records(CurrentFrom)
  Self.ThumbPlace(CurrentUse, CurrentFrom)
  Post(Event:NewSelection, CurrentUse)
  Return

!
!  Process the down scroll/arrow keys for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
TableHandleKeys.ListScrollDown  Procedure(CurrentUse, CurrentFrom)
  CODE
!
!  Move the selected record down 1 to the end of the queue.
!  Adjust the thumb placement.
!
  If CurrentUse{Prop:Selected} + 1 > Records(CurrentFrom) Then
    CurrentUse{Prop:Selected} = Records(CurrentFrom)
  Else
    CurrentUse{Prop:Selected} = CurrentUse{Prop:Selected} + 1
  End
  Self.ThumbPlace(CurrentUse, CurrentFrom)
  Post(Event:NewSelection, CurrentUse)
  Return

!
!  Process the home/top keys for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
TableHandleKeys.ListScrollTop  Procedure(CurrentUse, CurrentFrom)
  CODE
!
!  Move the selected record down to the beginning of the queue.
!  Adjust the thumb placement.
!
  CurrentUse{Prop:Selected} = 1
  Self.ThumbPlace(CurrentUse, CurrentFrom)
  Post(Event:NewSelection, CurrentUse)
  Return

!
!  Process the up keys for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
TableHandleKeys.ListScrollUp  Procedure(CurrentUse, CurrentFrom)
  CODE
!
!  Move the selected record up 1 to the beginning of the queue.
!  Adjust the thumb placement.
!
  CurrentUse{Prop:Selected} = CurrentUse{Prop:Selected} - 1
  If CurrentUse{Prop:Selected} < 1 Then
    CurrentUse{Prop:Selected} = 1
  End
  Self.ThumbPlace(CurrentUse, CurrentFrom)
  Post(Event:NewSelection, CurrentUse)
  Return

!
!  Process the thumb movement for the list control
!  Uses:
!    CurrentUse  - The use value used to reference the table on the window
!    CurrentFrom - The data queue
!
TableHandleKeys.ListScrollDrag  Procedure(CurrentUse, CurrentFrom)
  CODE
!
!  Move the selected record to the record matching the position of the thumb.
!    The help file had said that the high number is 255, but it seems that on 
!    basic list controls, it is set to the length of  the queue.
!
  !CurrentUse{Prop:Selected} = Records(CurrentFrom) * (CurrentUse{Prop:VScrollPos} / 255)
  CurrentUse{Prop:Selected} = CurrentUse{Prop:VScrollPos}
  If CurrentUse{Prop:Selected} = 0 Then
    CurrentUse{Prop:Selected} = 1
  End
  Post(Event:NewSelection, CurrentUse)